'NOTE: The code for the class are identical to the code in LateBindInt.vb file. The difference lies in
'attributes and interface explicitly defined.

'When enabling early binding, we need to also import the InteropServices as well the reference library.
Imports System.Runtime.InteropServices
Imports Oyster.Math

'Namespace is optional, but good idea to specify one, especially if we do not strong name our assembly.
'Without a strong name, two assembly with same namespace will conflict with each other.
Namespace IntXCOM

    'For early binding, we have to explicitly define our own interface to expose methods and the parameters.
    'We do not put in any actual code; we only need to list what we want to expose. The class will take
    'care of the implementation details.
    '
    'Also, we need to set attributes, which are enclosed in <>s to inform compiler how we want to build
    'interface and class.
    '
    'Two attributes need to be set: GUID and Interface type.
    '   GUID can be generated with GUIDgen.exe, which you can download from MSDN.
    '   ComInterfaceType; for most cases, we want Dual to enable both early and late binding.
    'Note this is a one statement with line continuations.
    <Guid("55C53C76-9D39-4673-9A81-E4289E5BC6DB"), _
        InterfaceType(ComInterfaceType.InterfaceIsDual)> _
    Public Interface IBigInt

        'To actually display the method in Access's object browser and intellisense, we must specify
        'attribute DispID
        <DispId(1)> Function conv(ByVal bigint As String, ByVal inbase As Byte, ByVal outbase As Byte) As String
        <DispId(2)> Function add(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String
        <DispId(3)> Function subtract(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String
        <DispId(4)> Function multiply(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String
        <DispId(5)> Function divide(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String
        <DispId(6)> Function modulo(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String
        <DispId(7)> Function badadd(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As UInteger) As String

    End Interface

    'Three attributes need to be defined for the class:
    '   GUID: Shoudl be another GUID generated from the same program as above. Do not use same GUID as interface
    '   Intertype: We specify "None" to force COM to use our own interface.
    '   ProgID: This is human-readable name that will be used in COM specifications
    <Guid("B6CB2887-4190-47ab-84D8-5DBF38382572"), _
         ClassInterface(ClassInterfaceType.None), _
         ProgId("BigIntX")> _
    Public Class BigInt
        'We must implement the interface. This is a separate statement.
        Implements IBigInt

        Private foo As IntX
        Private bar As IntX
        Private iter As IntX

        'Each method must likewise implement the method listed in the interface.
        'Note this is actually a single statement with a line continuation character.
        Public Function conv(ByVal bigint As String, ByVal inbase As Byte, ByVal outbase As Byte) As String _
        Implements IBigInt.conv

            foo = IntX.Parse(bigint, inbase)
            Return foo.ToString(outbase)

        End Function

        Public Function add(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String _
        Implements IBigInt.add

            foo = IntX.Parse(bigint1, nbase)
            bar = IntX.Parse(bigint2, nbase)
            iter = foo + bar
            Return iter.ToString(nbase)

        End Function

        Public Function subtract(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String _
        Implements IBigInt.subtract

            foo = IntX.Parse(bigint1, nbase)
            bar = IntX.Parse(bigint2, nbase)
            iter = foo - bar
            Return iter.ToString(nbase)

        End Function

        Public Function multiply(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String _
        Implements IBigInt.multiply

            foo = IntX.Parse(bigint1, nbase)
            bar = IntX.Parse(bigint2, nbase)
            iter = foo * bar
            Return iter.ToString(nbase)

        End Function

        Public Function divide(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String _
        Implements IBigInt.divide

            foo = IntX.Parse(bigint1, nbase)
            bar = IntX.Parse(bigint2, nbase)
            iter = foo / bar
            Return iter.ToString(nbase)

        End Function

        Public Function modulo(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String _
        Implements IBigInt.modulo

            foo = IntX.Parse(bigint1, nbase)
            bar = IntX.Parse(bigint2, nbase)
            iter = foo Mod bar
            Return iter.ToString(nbase)

        End Function

        Public Function badadd(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As UInteger) As String _
        Implements IBigInt.badadd

            foo = IntX.Parse(bigint1, nbase)
            bar = IntX.Parse(bigint2, nbase)
            iter = foo + bar
            Return iter.ToString(nbase)

        End Function

    End Class

End Namespace
