'Imports <namespace> indicates that we want to reference a certain library
'To create a reference, click Project -> Add Reference and click "Browse" tab to include the 
'library you wnat to use, then import it with the name as below.
Imports Oyster.Math

'This is the bare minimum required to create a wrapper class. Only late binding can be enabled. When
'this is registered for COM compatibility, compiler will automatically create an empty interface, GUIDs
'and all other details as required by COM specifications for you. 
Public Class LateBindInt

    'For this particular use, we do not want to expose custom data type to VBA or Jet as they are picky
    'with how those can be handled. Therefore, all methods below accept strings as an argument, parse it
    'to the custom data type then perform the operations supported by the custom data type and return the 
    'string back to Access environment.
    Private foo As IntX
    Private bar As IntX
    Private iter As IntX

    'Note the data type Byte is used instead of library's data type UInteger; UInteger is not available 
    'or supported in Access environment so we need to substitute with a compatible data type.

    'To Determine the data types a library expect, use Object Browser (press F2) to examine its methods
    'and parameters required.
    Public Function conv(ByVal bigint As String, ByVal inbase As Byte, ByVal outbase As Byte) As String

        foo = IntX.Parse(bigint, inbase)

        'Note that we return the function, rather than assigning the result to the function name in VBA.
        Return foo.ToString(outbase)

    End Function

    'This is an example of what happens when we use incompatible data type. In Access's Object Browser
    'You will note that it says <unsupported variant type> and the method cannot be called at all.
    Public Function badadd(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As UInteger) As String

        foo = IntX.Parse(bigint1, nbase)
        bar = IntX.Parse(bigint2, nbase)
        iter = foo + bar
        Return iter.ToString(nbase)

    End Function

    'Another thing to note is that while we may be writing in Visual Basic, we still have to know a bit
    'about how C# people may write their code. One example is overloading the operator; the author has
    'written a function altering the '+' operator to enable adding two custom data type 'IntX' just as
    'we would do with any numeric data types. 
    Public Function add(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String

        foo = IntX.Parse(bigint1, nbase)
        bar = IntX.Parse(bigint2, nbase)
        iter = foo + bar
        Return iter.ToString(nbase)

    End Function

    Public Function subtract(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String

        foo = IntX.Parse(bigint1, nbase)
        bar = IntX.Parse(bigint2, nbase)
        iter = foo - bar
        Return iter.ToString(nbase)

    End Function

    Public Function multiply(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String

        foo = IntX.Parse(bigint1, nbase)
        bar = IntX.Parse(bigint2, nbase)
        iter = foo * bar
        Return iter.ToString(nbase)

    End Function

    Public Function divide(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String

        foo = IntX.Parse(bigint1, nbase)
        bar = IntX.Parse(bigint2, nbase)
        iter = foo / bar
        Return iter.ToString(nbase)

    End Function

    Public Function modulo(ByVal bigint1 As String, ByVal bigint2 As String, ByVal nbase As Byte) As String

        foo = IntX.Parse(bigint1, nbase)
        bar = IntX.Parse(bigint2, nbase)
        iter = foo Mod bar
        Return iter.ToString(nbase)

    End Function

End Class